@@ -29,6 +29,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include "debug.h"
+#include "decoder.h"
 #include "utils.h"
 
 
@@ -191,6 +192,8 @@ move_ringtones (const gchar *new_ringtone_path,
         goto finally;
     }
 
+    ringtoned_decoder_schedule_if_needed (new_ringtone_path);
+
 finally:
     if (update)
     {
@@ -220,6 +223,8 @@ finally:
         g_free (new_ringtoned_cached_path);
     }
 
+    ringtoned_decoder_decode_pending ();
+
     if (io_channel)
         g_io_channel_unref (io_channel);
 }
