@@ -21,9 +21,11 @@
 
 #include "ringtone-chooser.h"
 #include <ctype.h>
+#include <gst/gst.h>
 #include <hildon/hildon.h>
 #include <hildon/hildon-fm.h>
 #include <libringtoned/ringtoned.h>
+#include <pulse/pulseaudio.h>
 #include <string.h>
 #include "contact.h"
 
@@ -48,7 +50,9 @@ struct _PCRRingtoneChooserPrivate
     GtkWidget *selector;
 
     gchar *ringtone;
-    RingtonedPlayer *player;
+
+    gboolean playing;
+    GstElement *playbin;
 };
 
 #define PCR_RINGTONE_CHOOSER_GET_PRIVATE(o) \
@@ -103,23 +107,76 @@ finally:
     return success;
 }
 
-static void
-stop_preview (PCRRingtoneChooser *self)
+static gboolean
+bus_watch_cb (GstBus     *bus,
+              GstMessage *msg,
+              gpointer    user_data)
 {
-    if (self->priv->player)
-    {
-        DEBUG ("Stop current preview");
+    PCRRingtoneChooser *self = user_data;
 
-        ringtoned_player_stop (self->priv->player);
-        g_object_unref (self->priv->player);
-        self->priv->player = NULL;
+    switch (GST_MESSAGE_TYPE (msg))
+    {
+        case GST_MESSAGE_EOS:
+            DEBUG ("Finished playing %s");
+            self->priv->playing = FALSE;
+            break;
+
+        case GST_MESSAGE_ERROR:
+        {
+            GError *err = NULL;
+            gchar *dbg_info = NULL;
+
+            gst_message_parse_error (msg, &err, &dbg_info);
+
+            g_printerr ("Error from element %s: %s\n",
+                    GST_OBJECT_NAME (msg->src), err->message);
+            g_printerr ("Debugging info:\n%s\n",
+                    dbg_info ? dbg_info : "none");
+
+            self->priv->playing = FALSE;
+
+            g_error_free (err);
+            g_free (dbg_info);
+
+            break;
+        }
+
+        default:
+            break;
     }
+
+    return TRUE;
 }
 
 static void
-selector_changed_cb (HildonTouchSelector *widget,
-                     gint                 column,
-                     PCRRingtoneChooser  *self)
+start_file (PCRRingtoneChooser *self,
+            const gchar        *path)
+{
+    gchar *escaped;
+    gchar *uri;
+
+    DEBUG ("Starting preview of %s", path);
+
+    if (self->priv->playing)
+    {
+        g_warning ("Already playing");
+        return;
+    }
+
+    escaped = g_uri_escape_string (path, "/", FALSE);
+    uri = g_strconcat ("file://", escaped, NULL);
+    g_object_set (self->priv->playbin, "uri", uri, NULL);
+
+    gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
+
+    self->priv->playing = TRUE;
+
+    g_free (uri);
+    g_free (escaped);
+}
+
+static void
+start_preview (PCRRingtoneChooser *self)
 {
     GtkTreeIter iter;
     gchar *preview_path;
@@ -127,8 +184,6 @@ selector_changed_cb (HildonTouchSelector *widget,
     if (!get_selected_iter (self, &iter))
         return;
 
-    stop_preview (self);
-
     gtk_tree_model_get (GTK_TREE_MODEL (self->priv->store), &iter,
             COL_PATH, &preview_path, -1);
 
@@ -137,15 +192,32 @@ selector_changed_cb (HildonTouchSelector *widget,
     else if (g_strcmp0 (preview_path, PCR_RINGTONE_SILENT) == 0)
         DEBUG ("Not playing preview for silent");
     else
-    {
-        DEBUG ("Starting preview of %s", preview_path);
-        self->priv->player = ringtoned_player_new (preview_path, TRUE);
-        ringtoned_player_start (self->priv->player);
-    }
+        start_file (self, preview_path);
 
     g_free (preview_path);
 }
 
+static void
+stop_preview (PCRRingtoneChooser *self)
+{
+    if (self->priv->playing)
+    {
+        DEBUG ("Stop current preview");
+
+        gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
+        self->priv->playing = FALSE;
+    }
+}
+
+static void
+selector_changed_cb (HildonTouchSelector *widget,
+                     gint                 column,
+                     PCRRingtoneChooser  *self)
+{
+    stop_preview (self);
+    start_preview (self);
+}
+
 static void
 pcr_ringtone_chooser_size_request (GtkWidget      *widget,
                                    GtkRequisition *requisition)
@@ -505,6 +577,32 @@ fill_store (PCRRingtoneChooser *self)
     ensure_current_ringtone (self);
 }
 
+static void
+init_playbin (PCRRingtoneChooser *self)
+{
+    pa_proplist *proplist;
+    GstElement *sink;
+    GstBus *bus;
+
+    /* Magical bits to make the ringtones ignore the system volume */
+    proplist = pa_proplist_new();
+    pa_proplist_sets (proplist, "event.id", "ringtone-preview");
+    pa_proplist_sets (proplist, "module-stream-restore.id",
+        "x-maemo-applet-profiles");
+
+    sink = gst_element_factory_make ("pulsesink", "ringtone-sink");
+    g_object_set (sink, "proplist", proplist, NULL);
+
+    self->priv->playbin = gst_element_factory_make ("playbin2",
+        "ringtone-playbin");
+    g_object_set (self->priv->playbin, "audio-sink", sink, NULL);
+
+    bus = gst_pipeline_get_bus (GST_PIPELINE (self->priv->playbin));
+    gst_bus_add_watch (bus, bus_watch_cb, self);
+
+    gst_object_unref (bus);
+}
+
 static void
 pcr_ringtone_chooser_init (PCRRingtoneChooser *self)
 {
@@ -528,7 +626,7 @@ pcr_ringtone_chooser_init (PCRRingtoneChooser *self)
 
     gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (self))),
             self->priv->selector, TRUE, TRUE, 0);
-} 
+}
 
 static void
 pcr_ringtone_chooser_constructed (GObject *object)
@@ -536,6 +634,7 @@ pcr_ringtone_chooser_constructed (GObject *object)
     PCRRingtoneChooser *self = PCR_RINGTONE_CHOOSER (object);
 
     fill_store (self);
+    init_playbin (self);
 
     /* We do this after the current ringtone was selected to avoid playing
      * anything as soon as we create the dialog */
@@ -551,6 +650,8 @@ pcr_ringtone_chooser_finalize (GObject *object)
     PCRRingtoneChooser *self = PCR_RINGTONE_CHOOSER (object);
 
     stop_preview (self);
+    if (self->priv->playbin)
+        gst_object_unref (self->priv->playbin);
 
     g_free (self->priv->ringtone);
 
